@@ -4,15 +4,21 @@ module Test;
 ## This is a temporary Test.pm to get us started until we get pugs's Test.pm
 ## working. It's shamelessly stolen & adapted from MiniPerl6 in the pugs repo.
 
-# variables to keep track of our tests
-my $num_of_tests_run    = 0;
-my $num_of_tests_failed = 0;
-my $todo_upto_test_num  = 0;
-my $todo_reason         = '';
-my $num_of_tests_planned;
-my $no_plan = 1;
-my $die_on_fail;
+# settable from outside
 my $perl6_test_times = ? %*ENV<PERL6_TEST_TIMES>;
+
+# global state
+my @vars;
+my $indents = "";
+
+# variables to keep track of our tests
+my $num_of_tests_run;
+my $num_of_tests_failed;
+my $todo_upto_test_num;
+my $todo_reason;
+my $num_of_tests_planned;
+my $no_plan;
+my $die_on_fail;
 my $time_before;
 my $time_after;
 
@@ -21,6 +27,8 @@ my $time_after;
 ## plans and don't call done_testing.
 my $done_testing_has_been_run = 0;
 
+# make sure we have initializations
+_init_vars();
 
 ## test functions
 
@@ -39,6 +47,7 @@ multi sub plan($number_of_tests) is export {
         $num_of_tests_planned = $number_of_tests;
         $no_plan = 0;
 
+        print $indents;
         say '1..' ~ $number_of_tests;
     }
     # Get two successive timestamps to say how long it takes to read the
@@ -48,8 +57,10 @@ multi sub plan($number_of_tests) is export {
     # lot slower than the non portable nqp::p6box_n(nqp::time_n).
     $time_before = nqp::p6box_n(nqp::time_n);
     $time_after  = nqp::p6box_n(nqp::time_n);
-    say '# between two timestamps ' ~
-        ceiling(($time_after-$time_before)*1_000_000) ~ ' microseconds'
+    print $indents
+      ~ '# between two timestamps '
+      ~ ceiling(($time_after-$time_before)*1_000_000) ~ ' microseconds'
+      ~ "\n"
         if $perl6_test_times;
     # Take one more reading to serve as the begin time of the first test
     $time_before = nqp::p6box_n(nqp::time_n);
@@ -135,8 +146,22 @@ sub skip_rest($reason = '<unknown>') is export {
     $time_before = nqp::p6box_n(nqp::time_n);
 }
 
+sub subtest(&subtests, $desc = '') is export {
+    _push_vars();
+    _init_vars();
+    $indents ~= "    ";
+    subtests();
+    done() if !$done_testing_has_been_run;
+    my $status =
+      $num_of_tests_failed == 0 && $num_of_tests_planned == $num_of_tests_run;
+    _pop_vars;
+    $indents = $indents.chop(4);
+    proclaim($status,$desc);
+}
+
 sub diag(Mu $message) is export {
     $time_after = nqp::p6box_n(nqp::time_n);
+    print $indents;
     say $message.Str.subst(rx/^^/, '# ', :g);
     $time_before = nqp::p6box_n(nqp::time_n);
 }
@@ -228,6 +253,45 @@ multi sub is_deeply(Mu $got, Mu $expected, $reason = '') is export
     return $ok;
 }
 
+sub throws_like($code, $ex_type, *%matcher) is export {
+    subtest {
+        plan 2 + %matcher.keys;
+        my $msg;
+        if $code ~~ Callable {
+            $msg = 'code dies';
+            $code()
+        } else {
+            $msg = "'$code' died";
+            EVAL $code;
+        }
+        ok 0, $msg;
+        skip 'Code did not die, can not check exception', 1 + %matcher.elems;
+        CATCH {
+            default {
+                ok 1, $msg;
+                my $type_ok = $_ ~~ $ex_type;
+                ok $type_ok , "right exception type ({$ex_type.^name})";
+                if $type_ok {
+                    for %matcher.kv -> $k, $v {
+                        my $got = $_."$k"();
+                        my $ok = $got ~~ $v,;
+                        ok $ok, ".$k matches {$v.defined ?? $v !! $v.gist}";
+                        unless $ok {
+                            diag "Got:      $got";
+                            diag "Expected: $v";
+                        }
+                    }
+                } else {
+                    diag "Got:      {$_.WHAT.gist}";
+                    diag "Expected: {$ex_type.gist}";
+                    diag "Exception message: $_.message()";
+                    skip 'wrong exception type', %matcher.elems;
+                }
+            }
+        }
+    }, "did we throws_like {$ex_type.^name}?";
+}
+
 sub _is_deeply(Mu $got, Mu $expected) {
     $got eqv $expected;
 }
@@ -246,18 +310,25 @@ sub proclaim($cond, $desc) {
     # exclude the time spent in proclaim from the test time
     $num_of_tests_run = $num_of_tests_run + 1;
 
+    print $indents;
     unless $cond {
         print "not ";
         unless  $num_of_tests_run <= $todo_upto_test_num {
             $num_of_tests_failed = $num_of_tests_failed + 1
         }
     }
-    print "ok ", $num_of_tests_run, " - ", $desc;
     if $todo_reason and $num_of_tests_run <= $todo_upto_test_num {
-        print $todo_reason;
+        # TAP parsers do not like '#' in the description, they'd miss the '# TODO'
+        print "ok ", $num_of_tests_run, " - ", $desc.subst('#', '', :g), $todo_reason;
+    }
+    else {
+        print "ok ", $num_of_tests_run, " - ", $desc;
     }
     print "\n";
-    print "# t=" ~ ceiling(($time_after-$time_before)*1_000_000) ~ "\n"
+    print $indents
+      ~ "# t="
+      ~ ceiling(($time_after-$time_before)*1_000_000)
+      ~ "\n"
         if $perl6_test_times;
 
     if !$cond && $die_on_fail && !$todo_reason {
@@ -277,6 +348,7 @@ sub done() is export {
 
     if $no_plan {
         $num_of_tests_planned = $num_of_tests_run;
+        print $indents;
         say "1..$num_of_tests_planned";
     }
 
@@ -288,6 +360,49 @@ sub done() is export {
     }
 }
 
+sub _init_vars {
+    $num_of_tests_run     = 0;
+    $num_of_tests_failed  = 0;
+    $todo_upto_test_num   = 0;
+    $todo_reason          = '';
+    $num_of_tests_planned = Any;
+    $no_plan              = 1;
+    $die_on_fail          = Any;
+    $time_before          = Any;
+    $time_after           = Any;
+    $done_testing_has_been_run = 0;
+}
+
+sub _push_vars {
+    @vars.push: [
+      $num_of_tests_run,
+      $num_of_tests_failed,
+      $todo_upto_test_num,
+      $todo_reason,
+      $num_of_tests_planned,
+      $no_plan,
+      $die_on_fail,
+      $time_before,
+      $time_after,
+      $done_testing_has_been_run,
+    ];
+}
+
+sub _pop_vars {
+    (
+      $num_of_tests_run,
+      $num_of_tests_failed,
+      $todo_upto_test_num,
+      $todo_reason,
+      $num_of_tests_planned,
+      $no_plan,
+      $die_on_fail,
+      $time_before,
+      $time_after,
+      $done_testing_has_been_run,
+    ) = @(@vars.pop);
+}
+
 END {
     ## In planned mode, people don't necessarily expect to have to call done
     ## So call it for them if they didn't
@@ -296,4 +411,34 @@ END {
     }
 }
 
+=begin pod
+
+=head1 NAME
+
+Test - Rakudo Testing Library
+
+=head1 SYNOPSIS
+
+  use Test;
+
+=head1 DESCRIPTION
+
+=head1 FUNCTIONS
+
+=head2 throws_like($code, Mu $expected_type, *%matchers)
+
+If C<$code> is C<Callable>, calls it, otherwise C<EVAL>s it,
+and expects it thrown an exception.
+
+If an exception is thrown, it is compared to C<$expected_type>.
+
+Then for each key in C<%matchers>, a method of that name is called
+on the resulting exception, and its return value smart-matched against
+the value.
+
+Each step is counted as a separate test; if one of the first two fails,
+the rest of the tests are skipped.
+
+=end pod
+
 # vim: ft=perl6
