@@ -3,6 +3,7 @@ my class Pair { ... }
 my class Range { ... }
 my class X::Bind::Slice { ... }
 my class X::Bind::ZenSlice { ... }
+my class X::Subscript::FromEnd { ... }
 
 my class Any { # declared in BOOTSTRAP
     # my class Any is Mu {
@@ -57,6 +58,7 @@ my class Any { # declared in BOOTSTRAP
     method end()   { self.list.end }
     method uniq(|c) { self.list.uniq(|c) }
     method squish(|c) { self.list.squish(|c) }
+    method rotor(|c) { self.list.rotor(|c) }
     method pick($n = 1) { self.list.pick($n) }
     method roll($n = 1) { self.list.roll($n) }
     method reverse() { self.list.reverse }
@@ -89,8 +91,10 @@ my class Any { # declared in BOOTSTRAP
     method lol()  {
         MapIter.new(self.list, { .item }, Mu).list
     }
-    method map($block) is rw {
-        MapIter.new(self, $block, Bool::True).list
+    proto method map (|) { * }
+    multi method map(Whatever) is rw { self }
+    multi method map($block, :$label) is rw {
+        MapIter.new(self, $block, Bool::True, :$label).list
     }
     method flatmap($block) is rw { flatmap($block, self) }
     method duckmap($block) is rw { duckmap($block, self) }
@@ -115,22 +119,87 @@ my class Any { # declared in BOOTSTRAP
     # auto-vivifying
     proto method push(|) { * }
     multi method push(Any:U \SELF: *@values) {
-        &infix:<=>(SELF, Array.new);
+        SELF = Array.new;
         SELF.push(@values);
     }
 
     proto method unshift(|) { * }
     multi method unshift(Any:U \SELF: *@values) {
-        &infix:<=>(SELF, Array.new);
+        SELF = Array.new;
         SELF.unshift(@values);
     }
 
-    method grep(Mu $test) is rw {
+    proto method grep(|) { * }
+    multi method grep(Regex:D $test) is rw {
+        self.map({ $_ if .match($test) });
+    }
+    multi method grep(Callable:D $test) is rw {
+        self.map({ $_ if $test($_) });
+    }
+    multi method grep(Mu $test) is rw {
         self.map({ $_ if $_ ~~ $test });
     }
-    method first(Mu $test) is rw {
-        my @results := self.grep($test);
-        @results ?? @results[0] !! Nil;
+
+    proto method grep-index(|) { * }
+    multi method grep-index(Regex:D $test) {
+        my $index = -1;
+        self.map: { $index++; +$index if .match($test) };
+    }
+    multi method grep-index(Callable:D $test) {
+        my $index = -1;
+        self.map: { $index++; +$index if $test($_) };
+    }
+    multi method grep-index(Mu $test) {
+        my $index = -1;
+        self.map: { $index++; +$index if $_ ~~ $test };
+    }
+
+    proto method first(|) { * }
+    multi method first(Regex:D $test) is rw {
+        self.map({ return $_ if .match($test) });
+        Nil;
+    }
+    multi method first(Callable:D $test) is rw {
+        self.map({ return $_ if $test($_) });
+        Nil;
+    }
+    multi method first(Mu $test) is rw {
+        self.map({ return $_ if $_ ~~ $test });
+        Nil;
+    }
+
+    proto method first-index(|) { * }
+    multi method first-index(Regex:D $test) {
+        my $index = -1;
+        self.map: { $index++; return $index if .match($test) };
+        Nil;
+    }
+    multi method first-index(Callable:D $test) {
+        my $index = -1;
+        self.map: { $index++; return $index if $test($_) };
+        Nil;
+    }
+    multi method first-index(Mu $test) {
+        my $index = -1;
+        self.map: { $index++; return $index if $_ ~~ $test };
+        Nil;
+    }
+
+    proto method last-index(|) { * }
+    multi method last-index(Regex:D $test) {
+        my $index = self.elems;
+        self.reverse.map: { $index--; return $index if .match($test) };
+        Nil;
+    }
+    multi method last-index(Callable:D $test) {
+        my $index = self.elems;
+        self.reverse.map: { $index--; return $index if $test($_) };
+        Nil;
+    }
+    multi method last-index(Mu $test) {
+        my $index = self.elems;
+        self.reverse.map: { $index--; return $index if $_ ~~ $test };
+        Nil;
     }
 
     method join($separator = '') {
@@ -145,7 +214,7 @@ my class Any { # declared in BOOTSTRAP
         while $list.gimme(0) {
             $tmp := $list.shift;
             nqp::push_s($rsa,
-              nqp::unbox_s(nqp::istype($tmp, Str) ?? $tmp !! $tmp.Str));
+              nqp::unbox_s(nqp::istype($tmp, Str) && nqp::isconcrete($tmp) ?? $tmp !! $tmp.Str));
         }
         nqp::push_s($rsa, '...') if $list.infinite;
         nqp::p6box_s(nqp::join(nqp::unbox_s($separator.Str), $rsa))
@@ -157,7 +226,7 @@ my class Any { # declared in BOOTSTRAP
         for self { 
             $min = $_ if .defined and !$min.defined || $cmp($_, $min) < 0;
         }
-        $min // +$Inf;
+        $min // Inf;
     }
 
     method max($by = &infix:<cmp>) {
@@ -166,7 +235,7 @@ my class Any { # declared in BOOTSTRAP
         for self { 
             $max = $_ if .defined and !$max.defined || $cmp($_, $max) > 0;
         }
-        $max // -$Inf;
+        $max // -Inf;
     }
 
     method minmax($by = &infix:<cmp>) {
@@ -200,8 +269,8 @@ my class Any { # declared in BOOTSTRAP
                 }
             }
         }
-        Range.new($min // +$Inf,
-                  $max // -$Inf,
+        Range.new($min // Inf,
+                  $max // -Inf,
                   :excludes_min($excludes_min),
                   :excludes_max($excludes_max));
     }
@@ -217,10 +286,14 @@ my class Any { # declared in BOOTSTRAP
     }
     multi method at_pos(Any:U \SELF: $pos) is rw {
         nqp::bindattr(my $v, Scalar, '$!whence',
-            -> { SELF.defined || &infix:<=>(SELF, Array.new);
+            -> { SELF.defined || (SELF = Array.new);
                  SELF.bind_pos($pos, $v) });
         $v
     }
+    proto method assign_pos(|) { * }
+    multi method assign_pos(\SELF: \pos, Mu \assignee) {
+        SELF.at_pos(pos) = assignee;
+    }
     
     method all() { all(self.list) }
     method any() { any(self.list) }
@@ -234,7 +307,7 @@ my class Any { # declared in BOOTSTRAP
     }
     multi method at_key(Any:U \SELF: $key) is rw {
         nqp::bindattr(my $v, Scalar, '$!whence',
-            -> { SELF.defined || &infix:<=>(SELF, Hash.new);
+            -> { SELF.defined || (SELF = Hash.new);
                  SELF.bind_key($key, $v) });
         $v
     }
@@ -243,10 +316,14 @@ my class Any { # declared in BOOTSTRAP
         fail "postcircumfix:<\{ \}> binding not defined for type {self.WHAT.perl}";
     }
     multi method bind_key(Any:U \SELF: $key, $BIND ) is rw {
-        &infix:<=>(SELF, Hash.new);
+        SELF = Hash.new;
         SELF.bind_key($key, $BIND);
         $BIND
     }
+    proto method assign_key(|) { * }
+    multi method assign_key(\SELF: \key, Mu \assignee) {
+        SELF.at_key(key) = assignee;
+    }
 
     method FLATTENABLE_LIST() { 
         my $list := self.list;
@@ -263,18 +340,20 @@ my class Any { # declared in BOOTSTRAP
 
     method KeySet() { DEPRECATED("'SetHash'"); self.SetHash }
     method KeyBag() { DEPRECATED("'BagHash'"); self.BagHash }
+
+    method Supply() { self.list.Supply }
 }
 Metamodel::ClassHOW.exclude_parent(Any);
 
 # builtin ops
 proto infix:<===>($?, $?) is pure { * }
-multi infix:<===>($a?)    { Bool::True }
+multi infix:<===>($?)    { Bool::True }
 multi infix:<===>($a, $b) {
     nqp::p6bool(nqp::iseq_s(nqp::unbox_s($a.WHICH), nqp::unbox_s($b.WHICH)))
 }
 
 proto infix:<before>($, $?)  is pure { * }
-multi infix:<before>($x?)      { Bool::True }
+multi infix:<before>($?)      { Bool::True }
 multi infix:<before>(\a, \b)   { (a cmp b) < 0 }
 
 proto infix:<after>($, $?) is pure { * }
@@ -321,22 +400,41 @@ multi infix:<minmax>(*@args) { @args.minmax }
 sub minmax(*@args, :&by = &infix:<cmp>) { @args.minmax(&by) }
 
 proto map(|) {*}
+# fails integration/99problems-21-to-30, test 12/13
+#multi map(&code, @values) { @values.map(&code) }
 multi map(&code, *@values) { @values.map(&code) }
+multi map(Whatever, \a)    { a }
 multi map(&code, Whatever) { (1..Inf).map(&code) }
 
 proto grep(|) {*}
+multi grep(Mu $test, @values) { @values.grep($test) }
 multi grep(Mu $test, *@values) { @values.grep($test) }
 
+proto grep-index(|) {*}
+multi grep-index(Mu $test, @values) { @values.grep-index($test) }
+multi grep-index(Mu $test, *@values) { @values.grep-index($test) }
+
 proto first(|) {*}
+multi first(Mu $test, @values) { @values.first($test) }
 multi first(Mu $test, *@values) { @values.first($test) }
 
+proto first-index(|) {*}
+multi first-index(Mu $test, @values) { @values.first-index($test) }
+multi first-index(Mu $test, *@values) { @values.first-index($test) }
+
+proto last-index(|) {*}
+multi last-index(Mu $test, @values) { @values.last-index($test) }
+multi last-index(Mu $test, *@values) { @values.last-index($test) }
+
 proto join(|) { * }
 multi join($sep = '', *@values) { @values.join($sep) }
 
 proto pick(|) { * }
+multi pick($n, @values) { @values.pick($n) }
 multi pick($n, *@values) { @values.pick($n) }
 
 proto roll(|) { * }
+multi roll($n, @values) { @values.roll($n) }
 multi roll($n, *@values) { @values.roll($n) }
 
 proto keys(|) { * }
@@ -377,12 +475,14 @@ multi squish(*@values, |c) { @values.squish(|c) }
 
 proto sub sort(|) {*}
 multi sub sort(*@values)      {
-    @values.at_pos(0).^does(Callable)
-        ?? do { my $cmp := @values.shift; @values.sort($cmp) }
-        !!  @values.sort;
+    nqp::istype(@values.at_pos(0), Callable)
+        ?? (my $cmp := @values.shift; @values.sort($cmp) )
+        !! @values.sort;
 }
 
 proto sub item(|) is pure { * }
+multi sub item(Iterable \x) { my $ = x }
+multi sub item(Parcel \x) { my $ = x }
 multi sub item(*@a) { my $ = @a }
 multi sub item(Mu $a) { $a }
 
@@ -418,7 +518,7 @@ sub SLICE_HUH ( \SELF, @nogo, %a, %adv ) is hidden_from_backtrace {
 
 # internal 1 element hash/array access with adverbs
 sub SLICE_ONE ( \SELF, $one, $array, *%adv ) is hidden_from_backtrace {
-    fail "Cannot use negative index $one on {SELF.WHAT.perl}"
+    fail X::Subscript::FromEnd.new(index => $one, type => SELF.WHAT)
       if $array && $one < 0;
 
     my $ex = SELF.can( $array ?? 'exists_pos' !! 'exists_key' )[0];
@@ -638,9 +738,8 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                     if !%a {                     # :delete:exists(0|1):kv(0|1)
                         $more.list.map( {
                             $de(SELF,$_) if $wasthere = $ex(SELF,$_);
-                            !$kv | $wasthere
-                              ?? ($_, !( $wasthere ?^ $exists ))
-                              !! ()
+                            ($_, !( $wasthere ?^ $exists )) 
+                              if !$kv | $wasthere;
                         } ).eager.Parcel
                     }
                     else {
@@ -652,9 +751,8 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                     if !%a {                     # :delete:exists(0|1):p(0|1)
                         $more.list.map( {
                             $de(SELF,$_) if $wasthere = $ex(SELF,$_);
-                            !$p | $wasthere
-                              ?? RWPAIR($_,!($wasthere ?^ $exists))
-                              !! ()
+                            RWPAIR($_,!($wasthere ?^ $exists))
+                              if !$p | $wasthere;
                         } ).eager.Parcel
                     }
                     else {
@@ -670,7 +768,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :delete:kv(0|1)
                     $kv
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? ( $_, $de(SELF,$_) ) !! ()
+                             ( $_, $de(SELF,$_) ) if $ex(SELF,$_);
                          } ).eager.Parcel
                       !! $more.list.map( {
                              ( $_, $de(SELF,$_) )
@@ -685,7 +783,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :delete:p(0|1)
                     $p
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? RWPAIR($_, $de(SELF,$_)) !! ()
+                             RWPAIR($_, $de(SELF,$_)) if $ex(SELF,$_);
                          } ).eager.Parcel
                       !! $more.list.map( {
                              RWPAIR($_, $de(SELF,$_))
@@ -700,7 +798,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :delete:k(0|1)
                     $k
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? ( $de(SELF,$_); $_ ) !! ()
+                             ( $de(SELF,$_); $_ ) if $ex(SELF,$_);
                          } ).eager.Parcel
                       !! $more.list.map( {
                              $de(SELF,$_); $_
@@ -715,7 +813,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :delete:v(0|1)
                     $v
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? $de(SELF,$_) !! ()
+                             $de(SELF,$_) if $ex(SELF,$_);
                      } ).eager.Parcel
                       !! $more.list.map( {
                              $de(SELF,$_)
@@ -739,7 +837,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :!delete?:exists(0|1):kv(0|1)
                     $kv
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? ( $_, $exists ) !! ()
+                             ( $_, $exists ) if $ex(SELF,$_);
                          } ).eager.Parcel
                       !! $more.list.map( {
                              ( $_, !( $ex(SELF,$_) ?^ $exists ) )
@@ -754,7 +852,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
                 if !%a {                       # :!delete?:exists(0|1):p(0|1)
                     $p
                       ?? $more.list.map( {
-                             $ex(SELF,$_) ?? RWPAIR( $_, $exists ) !! ()
+                             RWPAIR( $_, $exists ) if $ex(SELF,$_);
                          } ).eager.Parcel
                       !! $more.list.map( {
                              RWPAIR( $_, !( $ex(SELF,$_) ?^ $exists ) )
@@ -773,7 +871,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
             if !%a {                         # :!delete?:kv(0|1)
                 $kv
                   ?? $more.list.map( {
-                         $ex(SELF,$_) ?? ($_, $at(SELF,$_)) !! ()
+                         ($_, $at(SELF,$_)) if $ex(SELF,$_);
                      } ).eager.Parcel
                   !! $more.list.map( {
                          ($_, $at(SELF,$_))
@@ -788,7 +886,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
             if !%a {                         # :!delete?:p(0|1)
                 $p
                   ?? $more.list.map( {
-                         $ex(SELF,$_) ?? RWPAIR($_, $at(SELF,$_)) !! ()
+                         RWPAIR($_, $at(SELF,$_)) if $ex(SELF,$_);
                      } ).eager.Parcel
                   !! $more.list.map( {
                          RWPAIR( $_, $at(SELF,$_) )
@@ -802,7 +900,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
             my $k := %a.delete_key('k');
             if !%a {                         # :!delete?:k(0|1)
                 $k
-                  ?? $more.list.map( { $ex(SELF,$_) ?? $_ !! () } ).eager.Parcel
+                  ?? $more.list.map( { $_ if $ex(SELF,$_) } ).eager.Parcel
                   !! $more.list.eager.Parcel;
             }
             else {
@@ -814,7 +912,7 @@ sub SLICE_MORE ( \SELF, $more, $array, *%adv ) is hidden_from_backtrace {
             if !%a {                         # :!delete?:v(0|1)
                 $v
                   ??  $more.list.map( {
-                          $ex(SELF,$_) ?? $at(SELF,$_) !! ()
+                          $at(SELF,$_) if $ex(SELF,$_);
                       } ).eager.Parcel
                   !!  $more.list.map( {
                           $at(SELF,$_)
