@@ -1,18 +1,16 @@
 my class Bag does Baggy {
     has Int $!total;
     has $!WHICH;
+    has @!pairs;  # should be Parcel, but .Parcel doesn't do what we need
 
-    method total {
-        $!total //= [+] %!elems.values.map( { .value } );
-    }
-    submethod WHICH { $!WHICH }
-    submethod BUILD (:%elems)  {
-        my @keys := %elems.keys.sort;
-        $!WHICH  := self.^name
+    method total (--> Int) { $!total //= [+] self.values }
+
+    submethod WHICH {
+        $!WHICH //= self.^name
           ~ '|'
-          ~ @keys.map( { $_ ~ '(' ~ %elems{$_}.value ~ ')' } );
-        nqp::bindattr(self, Bag, '%!elems', %elems);
+          ~ %!elems.keys.sort.map( { $_ ~ '(' ~ %!elems{$_}.value ~ ')' } );
     }
+    submethod BUILD (:%!elems)  { }
 
     method at_key($k --> Int) {
         my $key := $k.WHICH;
@@ -20,9 +18,12 @@ my class Bag does Baggy {
           ?? %!elems{$key}.value
           !! 0;
     }
+    method pairs() {
+        @!pairs ||= %!elems.values.map: { Enum.new(:key(.key),:value(.value)) };
+    }
 
     method delete ($a --> Int) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :delete adverb");
+        DEPRECATED('the :delete adverb with postcircumfix:<{ }>');
         self.delete_key($a);
     }
     method delete_key($a --> Int) is hidden_from_backtrace {
