@@ -6,10 +6,11 @@ my role Baggy does QuantHash {
     method keys { %!elems.values.map( {.key} ) }
     method values { %!elems.values.map( {.value} ) }
     method kv { %!elems.values.map( {.key, .value} ) }
+    method kxxv { %!elems.values.map( {.key xx .value} ) }
     method elems(--> Int) { %!elems.elems }
     method total(--> Int) { [+] self.values }
     method exists ($k --> Bool) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :exists adverb");
+        DEPRECATED('the :exists adverb with postcircumfix:<{ }>');
         self.exists_key($k);
     }
     method exists_key($k --> Bool) {
@@ -73,7 +74,7 @@ my role Baggy does QuantHash {
     }
 
     method list() { self.keys }
-    method pairs() { %!elems.values }
+    method pairs() { %!elems.values.map: { (.key => .value) } }
 
     method grab ($count = 1) {
         my @grab = ROLLPICKGRAB(self, $count, %!elems.values);
@@ -97,7 +98,7 @@ my role Baggy does QuantHash {
         my $total = $self.total;
         my $todo  = $count ~~ Num
           ?? $total min $count
-          !! ($count ~~ Whatever ?? ( $keep ?? $Inf !! $total ) !! $count);
+          !! ($count ~~ Whatever ?? ( $keep ?? Inf !! $total ) !! $count);
 
         map {
             my $rand = $total.rand.Int;
