@@ -17,7 +17,7 @@ my class Code does Callable { # declared in BOOTSTRAP
     multi method Str(Code:D:) { self.name }
 
     method outer(Code:D:) {
-        nqp::getcodeobj(nqp::p6staticouter($!do))
+        nqp::ifnull(nqp::getcodeobj(nqp::p6staticouter($!do)), Mu)
     }
 
     # returns an identifier for this code object
@@ -26,3 +26,5 @@ my class Code does Callable { # declared in BOOTSTRAP
         nqp::p6box_i(nqp::where(nqp::getstaticcode($!do)));
     }
 }
+
+# vim: ft=perl6 expandtab sw=4
