@@ -39,6 +39,7 @@ my class Exception {
     }
 
     method resume() {
+#?if parrot
         my Mu $resume := nqp::atkey($!ex, 'resume');
         if $resume {
             $resume();
@@ -46,8 +47,14 @@ my class Exception {
         else {
             die "Exception is not resumable";
         }
+#?endif
+#?if !parrot
+        nqp::resume($!ex);
+        True
+#?endif
     }
 
+    method die(Exception:D:) { self.throw }
     method fail(Exception:D:) {
         try self.throw;
         my $fail := Failure.new($!);
@@ -66,6 +73,7 @@ my class X::AdHoc is Exception {
 }
 
 my class X::Method::NotFound is Exception {
+    has $.invocant;
     has $.method;
     has $.typename;
     has Bool $.private = False;
@@ -286,6 +294,22 @@ my class X::IO::Copy does X::IO is Exception {
     }
 }
 
+my class X::IO::DoesNotExist does X::IO is Exception {
+    has $.path;
+    has $.trying;
+    method message() {
+        "Failed to find '$.path' while trying to do '.$.trying'"
+    }
+}
+
+my class X::IO::NotAFile does X::IO is Exception {
+    has $.path;
+    has $.trying;
+    method message() {
+        "'$.path' is not a regular file while trying to do '.$.trying'"
+    }
+}
+
 my class X::IO::Symlink does X::IO is Exception {
     has $.target;
     has $.name;
@@ -363,11 +387,11 @@ my role X::Comp is Exception {
     has @.highexpect;
     multi method gist(::?CLASS:D: :$sorry = True, :$expect = True) {
         if $.is-compile-time {
-            my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*OS ne 'MSWin32';
+            my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*DISTRO.name ne 'mswin32';
             my ($red, $green, $yellow, $clear) = $color
                 ?? ("\e[31m", "\e[32m", "\e[33m", "\e[0m")
                 !! ("", "", "", "");
-            my $eject = $*OS eq 'MSWin32' ?? "<HERE>" !! "\x[23CF]";
+            my $eject = $*DISTRO.name eq 'MSWin32' ?? "<HERE>" !! "\x[23CF]";
             my $r = $sorry ?? self.sorry_heading() !! "";
             $r ~= "$.message\nat $.filename():$.line\n------> ";
             $r ~= "$green$.pre$yellow$eject$red$.post$clear" if defined $.pre;
@@ -378,9 +402,10 @@ my role X::Comp is Exception {
                 }
             }
             for @.modules.reverse[1..*] {
+                my $line = nqp::p6box_i($_<line>);
                 $r ~= $_<module>.defined
-                        ?? "\n  from module $_<module> ($_<filename>:$_<line>)"
-                        !! "\n  from $_<filename>:$_<line>";
+                        ?? "\n  from module $_<module> ($_<filename>:$line)"
+                        !! "\n  from $_<filename>:$line";
             }
             $r;
         }
@@ -389,7 +414,7 @@ my role X::Comp is Exception {
         }
     }
     method sorry_heading() {
-        my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*OS ne 'MSWin32';
+        my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*DISTRO.name ne 'mswin32';
         my ($red, $clear) = $color ?? ("\e[31m", "\e[0m") !! ("", "");
         "$red==={$clear}SORRY!$red===$clear Error while compiling $.filename\n"
     }
@@ -410,7 +435,7 @@ my class X::Comp::Group is Exception {
     multi method gist(::?CLASS:D:) {
         my $r = "";
         if $.panic || @.sorrows {
-            my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*OS ne 'MSWin32';
+            my $color = %*ENV<RAKUDO_ERROR_COLOR> // $*DISTRO.name ne 'mswin32';
             my ($red, $clear) = $color ?? ("\e[31m", "\e[0m") !! ("", "");
             $r ~= "$red==={$clear}SORRY!$red===$clear\n";
             for @.sorrows {
@@ -773,6 +798,14 @@ my class X::Bind::ZenSlice is X::Bind::Slice {
     }
 }
 
+my class X::Subscript::FromEnd is Exception {
+    has $.index;
+    has $.type;
+    method message() {
+        "Unsupported use of [{$.index}] subscript to access from end of {$.type.^name}; in Perl 6 please use [*{$.index}]"
+    }
+}
+
 my class X::Value::Dynamic does X::Comp {
     has $.what;
     method message() { "$.what value must be known at compile time" }
@@ -886,6 +919,10 @@ my class X::Syntax::Missing does X::Syntax {
     has $.what;
     method message() { "Missing $.what" }
 }
+my class X::Syntax::BlockGobbled does X::Syntax {
+    has $.what;
+    method message() { "{ $.what ?? 'Function ' ~ $.what !! 'Expression' } needs parens to avoid gobbling block" };
+}
 
 my class X::Syntax::Perl5Var does X::Syntax {
     has $.name;
@@ -919,7 +956,7 @@ my class X::Syntax::Perl5Var does X::Syntax {
       '$^I' => '$*INPLACE',
       '$^M' => 'a global form such as $*M',
       '$^N' => '$/[*-1]',
-      '$^O' => '$?OS or $*OS',
+      '$^O' => '$?DISTRO.name or $*DISTRO.name',
       '$^R' => 'an explicit result variable',
       '$^S' => 'context function',
       '$^T' => '$*BASETIME',
@@ -1066,7 +1103,7 @@ my class X::Constructor::Positional is Exception {
 }
 
 my class X::Hash::Store::OddNumber is Exception {
-    method message() { "Odd number of elements found where hash expected" }
+    method message() { "Odd number of elements found where hash initializer expected" }
 }
 
 my class X::Package::Stubbed does X::Comp {
@@ -1184,7 +1221,15 @@ my class X::TypeCheck is Exception {
 }
 
 my class X::TypeCheck::Binding is X::TypeCheck {
+    has $.symbol;
     method operation { 'binding' }
+    method message() {
+        if $.symbol {
+            "Type check failed in $.operation $.symbol; expected '{$.expected.^name}' but got '{$.got.^name}'";
+        } else {
+            "Type check failed in $.operation; expected '{$.expected.^name}' but got '{$.got.^name}'";
+        }
+    }
 }
 my class X::TypeCheck::Return is X::TypeCheck {
     method operation { 'returning' }
@@ -1210,7 +1255,7 @@ my class X::TypeCheck::Argument is X::TypeCheck {
             ($.protoguilt ?? "Calling proto of '" !! "Calling '") ~
             $.objname ~ "' " ~
             (+@.arguments == 0
-              ?? "requires arguments\n"
+              ?? "requires arguments (if you meant to operate on \$_, please use .$.objname or use an explicit invocant or argument)\n"
               !! "will never work with argument types (" ~ join(', ', @.arguments) ~ ")\n") 
             ~ $.signature 
     }
@@ -1225,8 +1270,9 @@ my class X::TypeCheck::Splice is X::TypeCheck does X::Comp {
 }
 
 my class X::Assignment::RO is Exception {
+    has $.typename = "value";
     method message {
-        "Cannot modify an immutable value";
+        "Cannot modify an immutable {$.typename}";
     }
 }
 
@@ -1422,17 +1468,17 @@ my class X::Caller::NotDynamic is Exception {
 
 {
     my %c_ex;
-    %c_ex{'X::TypeCheck::Binding'} := sub ($got, $expected) is hidden_from_backtrace {
-            X::TypeCheck::Binding.new(:$got, :$expected).throw;
+    %c_ex{'X::TypeCheck::Binding'} := sub (Mu $got, Mu $expected, $symbol?) is hidden_from_backtrace {
+            X::TypeCheck::Binding.new(:$got, :$expected, :$symbol).throw;
         };
-    %c_ex<X::TypeCheck::Assignment> := sub ($symbol, $got, $expected) is hidden_from_backtrace {
+    %c_ex<X::TypeCheck::Assignment> := sub (Mu $symbol, Mu $got, $expected) is hidden_from_backtrace {
             X::TypeCheck::Assignment.new(:$symbol, :$got, :$expected).throw;
         };
-    %c_ex{'X::TypeCheck::Return'} := sub ($got, $expected) is hidden_from_backtrace {
+    %c_ex{'X::TypeCheck::Return'} := sub (Mu $got, Mu $expected) is hidden_from_backtrace {
             X::TypeCheck::Return.new(:$got, :$expected).throw;
         };
-    %c_ex<X::Assignment::RO> := sub () is hidden_from_backtrace {
-            X::Assignment::RO.new.throw;
+    %c_ex<X::Assignment::RO> := sub ($typename = "value") is hidden_from_backtrace {
+            X::Assignment::RO.new(:$typename).throw;
         };
     %c_ex{'X::ControlFlow::Return'} := sub () is hidden_from_backtrace {
             X::ControlFlow::Return.new().throw;
@@ -1452,5 +1498,4 @@ my class X::Caller::NotDynamic is Exception {
     0;
 }
 
-
 # vim: ft=perl6 expandtab sw=4
