@@ -26,20 +26,18 @@ my class HashIter is Iterator {
     
     method reify($n?, :$sink) {  # hashes are finite, and hashiter non-reentrant, so do eager snapshot for now
         unless nqp::isconcrete($!reified) {
-            my int $i     =  0;
             my int $mode  =  $!mode;
             my Mu $rpa    := nqp::list();
             my $it        := $!hashiter;
             
             my Mu $pairish;
-            if $mode == 0 {
+            if $mode == 0 {   # :pairs
                 if nqp::defined($!keystore) {
                     while $it {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, Pair.new(
                             :key(nqp::atkey($!keystore, nqp::iterkey_s($pairish))),
                             :value(nqp::hllize(nqp::iterval($pairish)))));
-                        $i = $i + 1;
                     }
                 }
                 else {
@@ -48,17 +46,15 @@ my class HashIter is Iterator {
                         nqp::push($rpa, Pair.new(
                             :key(nqp::p6box_s(nqp::iterkey_s($pairish))),
                             :value(nqp::hllize(nqp::iterval($pairish)))));
-                        $i = $i + 1;
                     }
                 }
             }
-            elsif $mode == 1 {
+            elsif $mode == 1 {  # :kv
                 if nqp::defined($!keystore) {
                     while $it {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, nqp::atkey($!keystore, nqp::iterkey_s($pairish)).item);
                         nqp::push($rpa, nqp::hllize(nqp::iterval($pairish)).item);
-                        $i = $i + 1;
                     }
                 }
                 else {
@@ -66,41 +62,36 @@ my class HashIter is Iterator {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, nqp::p6box_s(nqp::iterkey_s($pairish)));
                         nqp::push($rpa, nqp::hllize(nqp::iterval($pairish)).item);
-                        $i = $i + 1;
                     }
                 }
             }
-            elsif $mode == 2 {
+            elsif $mode == 2 {  # :k
                 if nqp::defined($!keystore) {
                     while $it {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, nqp::atkey($!keystore, nqp::iterkey_s($pairish)).item);
-                        $i = $i + 1;
                     }
                 }
                 else {
                     while $it {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, nqp::p6box_s(nqp::iterkey_s($pairish)));
-                        $i = $i + 1;
                     }
                 }
             }
-            elsif $mode == 3 {
+            elsif $mode == 3 {  # :v
                 while $it {
                     $pairish := nqp::shift($it);
                     nqp::push($rpa, nqp::hllize(nqp::iterval($pairish)).item);
-                    $i = $i + 1;
                 }
             }
-            elsif $mode == 4 {
+            elsif $mode == 4 {  # :invert
                 if nqp::defined($!keystore) {
                     while $it {
                         $pairish := nqp::shift($it);
                         nqp::push($rpa, Pair.new(
                             :value(nqp::atkey($!keystore, nqp::iterkey_s($pairish))),
                             :key(nqp::hllize(nqp::iterval($pairish)))));
-                        $i = $i + 1;
                     }
                 }
                 else {
@@ -109,12 +100,11 @@ my class HashIter is Iterator {
                         nqp::push($rpa, Pair.new(
                             :value(nqp::p6box_s(nqp::iterkey_s($pairish))),
                             :key(nqp::hllize(nqp::iterval($pairish)))));
-                        $i = $i + 1;
                     }
                 }
             }
             else {
-                die "Unknown hash iteration mode";
+                die "Unknown hash iteration mode: $mode";
             }
 
             if $it {
