@@ -1,21 +1,53 @@
-my role IO { }
+my role IO {
+    method umask { state $ = EVAL "0o" ~ qx/umask/ }
+}
 
 sub print(|) {
     my $args := nqp::p6argvmarray();
-    $*OUT.print(nqp::shift($args)) while $args;
+    my $out := $*OUT;
+    $out.print(nqp::shift($args)) while $args;
     Bool::True
 }
 
-sub say(|) {
+proto sub say(|) { * }
+multi sub say(Obsolete:D \x) {
+    my $out := $*OUT;
+    $out.print(x.gist);
+    $out.print("\n");
+}
+multi sub say(Str:D \x) {
+    my $out := $*OUT;
+    $out.print(x);
+    $out.print("\n");
+}
+multi sub say(\x) {
+    my $out := $*OUT;
+    $out.print(x.gist);
+    $out.print("\n");
+}
+multi sub say(|) {
     my $args := nqp::p6argvmarray();
-    $*OUT.print(nqp::shift($args).gist) while $args;
-    $*OUT.print("\n");
+    my $out := $*OUT;
+    $out.print(nqp::shift($args).gist) while $args;
+    $out.print("\n");
 }
 
-sub note(|) {
+proto sub note(|) { * }
+multi sub note(Str:D \x) {
+    my $err := $*ERR;
+    $err.print(x);
+    $err.print("\n");
+}
+multi sub note(\x) {
+    my $err := $*ERR;
+    $err.print(x.gist);
+    $err.print("\n");
+}
+multi sub note(|) {
     my $args := nqp::p6argvmarray();
-    $*ERR.print(nqp::shift($args).gist) while $args;
-    $*ERR.print("\n");
+    my $err := $*ERR;
+    $err.print(nqp::shift($args).gist) while $args;
+    $err.print("\n");
 }
 
 sub gist(|) {
@@ -30,59 +62,89 @@ sub prompt($msg) {
 
 my role IO::FileTestable does IO {
     method d() {
-        self.e && nqp::p6bool(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)), 
-                                        nqp::const::STAT_ISDIR))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_ISDIR))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<d>)
     }
 
     method e() {
-        nqp::p6bool(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)), 
+        nqp::p6bool(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)),
                               nqp::const::STAT_EXISTS))
     }
 
     method f() {
-        self.e && nqp::p6bool(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)), 
-                              nqp::const::STAT_ISREG))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_ISREG))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<f>)
     }
 
     method s() {
-        self.e
-          && nqp::p6box_i( nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)),
-                                 nqp::const::STAT_FILESIZE) );
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_ISREG))
+            ?? nqp::p6box_i(nqp::stat($unboxed, nqp::const::STAT_FILESIZE))
+            !! fail X::IO::NotAFile.new(:path(self.Str),:trying<s>)
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<s>)
     }
 
     method l() {
-        nqp::p6bool(nqp::fileislink(IO::Spec.rel2abs(self.Str)))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::fileislink($unboxed))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<l>)
     }
 
     method r() {
-        nqp::p6bool(nqp::filereadable(IO::Spec.rel2abs(self.Str)))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::filereadable($unboxed))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<r>)
     }
 
     method w() {
-        nqp::p6bool(nqp::filewritable(IO::Spec.rel2abs(self.Str)))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::filewritable($unboxed))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<w>)
     }
 
     method x() {
-        nqp::p6bool(nqp::fileexecutable(IO::Spec.rel2abs(self.Str)))
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::fileexecutable($unboxed))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<x>)
     }
 
     method z() {
-        self.e && self.s == 0;
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_ISREG))
+            ?? nqp::p6box_i(nqp::stat($unboxed, nqp::const::STAT_FILESIZE)) == 0
+            !! fail X::IO::NotAFile.new(:path(self.Str),:trying<z>)
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<z>)
     }
 
     method modified() {
-         nqp::p6box_i(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)),
-                                nqp::const::STAT_MODIFYTIME));
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6box_i(nqp::stat($unboxed, nqp::const::STAT_MODIFYTIME))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<modified>)
     }
 
     method accessed() {
-         nqp::p6box_i(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)),
-                                nqp::const::STAT_ACCESSTIME));
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6box_i(nqp::stat($unboxed, nqp::const::STAT_ACCESSTIME))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<accessed>)
     }
 
     method changed() { 
-         nqp::p6box_i(nqp::stat(nqp::unbox_s(IO::Spec.rel2abs(self.Str)),
-                                nqp::const::STAT_CHANGETIME));
+        my Mu $unboxed := nqp::unbox_s(IO::Spec.rel2abs(self.Str));
+        nqp::p6bool(nqp::stat($unboxed, nqp::const::STAT_EXISTS))
+          ?? nqp::p6box_i(nqp::stat($unboxed, nqp::const::STAT_CHANGETIME))
+          !! fail X::IO::DoesNotExist.new(:path(self.Str),:trying<changed>)
     }
 }
 
@@ -93,9 +155,10 @@ my class IO::Handle does IO::FileTestable {
     has $.path;
 
     proto method open(|) { * }
-    multi method open($path? is copy, :$r, :$w, :$a, :$p, :$bin, :$chomp = Bool::True,
+    multi method open($path? is copy, :$r is copy, :$w is copy, :$rw, :$a, :$p, :$bin, :$chomp = Bool::True,
             :enc(:$encoding) = 'utf8') {
         $path //= $!path;
+        $r = $w = True if $rw;
         my $abspath = defined($*CWD) ?? IO::Spec.rel2abs($path) !! $path;
 #?if parrot
         my $mode =  $p ?? ($w ||  $a ?? 'wp' !! 'rp') !!
@@ -178,58 +241,44 @@ my class IO::Handle does IO::FileTestable {
         $c;
     }
 
-    method lines($limit = $Inf) {
-        my $count = 0;
-        gather while ++$count <= $limit && (my $line = self.get).defined {
-            take $line;
+    method lines($limit = Inf) {
+        if $limit == Inf {
+            gather while nqp::p6definite(my $line = self.get) {
+                take $line;
+            }
+        }
+        else {
+            my $count = 0;
+            gather while ++$count <= $limit && nqp::p6definite(my $line = self.get) {
+                take $line;
+            }
         }
     }
 
     method read(IO::Handle:D: Cool:D $bytes as Int) {
         my $buf := buf8.new();
-#?if parrot
-        # Relies on nqp::encode passing the binary encoding straight on down
-        # to Parrot.
-        my Mu $parrot_buffer := $!PIO.read_bytes(nqp::unbox_i($bytes));
-        nqp::encode($parrot_buffer.get_string('binary'), 'binary', $buf);
-#?endif
-#?if !parrot
         nqp::readfh($!PIO, $buf, nqp::unbox_i($bytes));
-#?endif
         $buf;
     }
+
     # second arguemnt should probably be an enum
     # valid values for $whence:
     #   0 -- seek from beginning of file
     #   1 -- seek relative to current position
     #   2 -- seek from the end of the file
     method seek(IO::Handle:D: Int:D $offset, Int:D $whence) {
-        $!PIO.seek(nqp::unbox_i($whence), nqp::unbox_i($offset));
+        nqp::seekfh($!PIO, $offset, $whence);
         True;
     }
+
     method tell(IO::Handle:D:) returns Int {
         nqp::p6box_i(
-#?if parrot
-            $!PIO.tell
-#?endif
-#?if !parrot
             nqp::tellfh($!PIO)
-#?endif
         );
     }
 
     method write(IO::Handle:D: Blob:D $buf) {
-#?if parrot
-        # This relies on the Parrot 'binary' encoding and that nqp::decode
-        # passes encoding straight down to Parrot.
-        my str $encoding = $!PIO.encoding;
-        $!PIO.encoding('binary');
-        $!PIO.print(nqp::decode(nqp::decont($buf), 'binary'));
-        $!PIO.encoding($encoding) unless $encoding eq 'binary';
-#?endif
-#?if !parrot
         nqp::writefh($!PIO, nqp::decont($buf));
-#?endif
         True;
     }
 
@@ -243,8 +292,8 @@ my class IO::Handle does IO::FileTestable {
 
 
     proto method print(|) { * }
-    multi method print(IO::Handle:D: Str:D $value) {
-        nqp::printfh($!PIO, nqp::unbox_s($value));
+    multi method print(IO::Handle:D: Str:D \x) {
+        nqp::printfh($!PIO, nqp::unbox_s(x));
         Bool::True
     }
     multi method print(IO::Handle:D: *@list) {
@@ -357,6 +406,10 @@ my class IO::Handle does IO::FileTestable {
     }
 }
 
+#?if moar
+my class IO::Notification { ... }
+#?endif
+
 my class IO::Path is Cool does IO::FileTestable {
     method SPEC { IO::Spec.MODULE };
     has Str $.path;
@@ -379,11 +432,11 @@ my class IO::Path is Cool does IO::FileTestable {
     }
 
     multi method new(:$basename!, :$directory = '.', :$volume = '') {
-        self.new: path=>$.SPEC.join($volume, $directory, $basename);
+        self.bless: path=>$.SPEC.join($volume, $directory, $basename);
     }
 
     multi method new(Str:D $path) {
-        self.new(:$path)
+        self.bless(:$path)
     }
 
     method path(IO::Path:D:) {
@@ -436,6 +489,12 @@ my class IO::Path is Cool does IO::FileTestable {
         open($!path, |%opts);
     }
 
+#?if moar
+    method watch(IO::Path:D:) {
+        IO::Notification.watch_path($!path);
+    }
+#?endif
+
     method is-absolute {
         $.SPEC.is-absolute($!path);
     }
@@ -507,7 +566,7 @@ my class IO::Path is Cool does IO::FileTestable {
         }
     }
 
-    method contents(IO::Path:D: Mu :$test = none('.', '..')) {
+    method contents(IO::Path:D: Mu :$test = { $_ ne '.' && $_ ne '..' }) {
 
         CATCH {
             default {
@@ -522,42 +581,42 @@ my class IO::Path is Cool does IO::FileTestable {
         my int $elems = nqp::elems($RSA);
         gather loop (my int $i = 0; $i < $elems; $i = $i + 1) {
             my Str $file := nqp::p6box_s(pir::trans_encoding__Ssi(
-			nqp::atpos_s($RSA, $i),
-			pir::find_encoding__Is('utf8')));
+              nqp::atpos_s($RSA, $i),
+              pir::find_encoding__Is('utf8')));
             if $file ~~ $test {
                 take self.child($file);
             }
         }
 #?endif
+#?if jvm
+        my $cwd_chars = $*CWD.chars;
+#?endif
 #?if !parrot
         my Mu $dirh := nqp::opendir(self.absolute.Str);
         my $next = 1;
         gather {
-#?endif
-#?if jvm
             take $_.path if $_ ~~ $test for ".", "..";
-#?endif
-#?if !parrot
+            my $SPEC = $.SPEC;
             loop {
-                my Str $elem := nqp::nextfiledir($dirh);
-                if nqp::isnull_s($elem) || !$elem {
+                my str $elem = nqp::nextfiledir($dirh);
+                if nqp::isnull_s($elem) || nqp::chars($elem) == 0 {
                     nqp::closedir($dirh);
                     last;
-                } else {
+                }
+                elsif $elem ne '.' | '..' {
 #?endif
 #?if jvm
                     # jvm's nextfiledir gives us absolute paths back, moar does not.
-                    $elem := $elem.substr($*CWD.chars + 1) if self.is-relative;
+                    $elem = nqp::substr($elem, $cwd_chars + 1) if self.is-relative;
 #?endif
 #?if moar
-                    next unless $elem ~~ $test;
-                    $elem := $.SPEC.catfile($!path, $elem) if self ne '.';
+                    $elem = $SPEC.catfile($!path, $elem) if $!path ne '.';
 #?endif
 #?if !parrot
-                    if $elem.substr(0, 2) eq any("./", ".\\") {
-                        $elem := $elem.substr(2);
+                    if nqp::substr($elem, 0, 2) eq "./" | ".\\" {
+                        $elem = nqp::substr($elem, 2);
                     }
-                    take $elem.path if $elem ~~ $test;
+                    take IO::Path.new($elem) if $test.ACCEPTS($elem);
                 }
             }
         }
@@ -605,12 +664,12 @@ sub rmdir($path as Str) {
 }
 
 proto sub open(|) { * }
-multi sub open($path, :$r, :$w, :$a, :$p, :$bin, :$chomp = Bool::True, :enc(:$encoding) = 'utf8') {
-    IO::Handle.new.open($path, :$r, :$w, :$a, :$p, :$bin, :$chomp, :$encoding);
+multi sub open($path, :$r is copy, :$w is copy, :$rw, :$a, :$p, :$bin, :$chomp = Bool::True, :enc(:$encoding) = 'utf8') {
+    IO::Handle.new.open($path, :$r, :$w, :$rw, :$a, :$p, :$bin, :$chomp, :$encoding);
 }
 
 proto sub lines(|) { * }
-multi sub lines($fh = $*ARGFILES, $limit = $Inf) { 
+multi sub lines($fh = $*ARGFILES, $limit = Inf) { 
     $fh.lines($limit) 
 }
 
@@ -672,14 +731,7 @@ multi sub spurt(Cool $filename,
     proto sub cwd(|) { * }
     multi sub cwd() {
         return nqp::p6box_s(
-#?if parrot
-            pir::trans_encoding__Ssi(
-                nqp::cwd(),
-                pir::find_encoding__Is('utf8'))
-#?endif
-#?if !parrot
-            nqp::cwd(),
-#?endif
+            nqp::cwd()
         );
         CATCH {
             default {
@@ -718,7 +770,7 @@ multi sub cwd() {
 proto sub chdir(|) { * }
 multi sub chdir(IO::Path:D $path) { chdir $path.Str }
 multi sub chdir($path as Str) {
-    my $newpath = IO::Path.new($path);
+    my $newpath = IO::Path.new(IO::Spec.canonpath($path));
     if $newpath.is-relative {
         my $tmp = $*CWD;
         for IO::Spec.splitdir($newpath) -> $segment {
