@@ -1,5 +1,6 @@
 # for our tantrums
 my class X::TypeCheck { ... }
+my role Supply { ... }
 
 my sub combinations($n, $k) {
     my @result;
@@ -9,25 +10,25 @@ my sub combinations($n, $k) {
 
     @stack.push(0);
     gather while @stack {
-	my $index = @stack - 1;
-	my $value = @stack.pop;
+        my $index = @stack - 1;
+        my $value = @stack.pop;
 
-	while $value < $n {
-	    @result[$index++] = $value++;
-	    @stack.push($value);
-	    if $index == $k {
-		take [@result];
-		$value = $n;  # fake a last
-	    }
-	}
+        while $value < $n {
+            @result[$index++] = $value++;
+            @stack.push($value);
+            if $index == $k {
+                take [@result];
+                $value = $n;  # fake a last
+            }
+        }
     }
 }
 
 my sub permutations(Int $n) {
     $n == 1 ?? ( [0,] ) !!
     gather for ^$n -> $i {
-	my @i = grep none($i), ^$n;
-	take [$i, @i[@$_]] for permutations($n - 1);
+        my @i = grep none($i), ^$n;
+        take [$i, @i[@$_]] for permutations($n - 1);
     }
 }
 
@@ -104,6 +105,8 @@ my class List does Positional { # declared in BOOTSTRAP
         nqp::p6parcel($rpa, Any);
     }
 
+    method Supply(List:D:) { Supply.for(self) }
+
     multi method at_pos(List:D: $pos is copy) is rw {
         $pos = $pos.Int;
         self.exists_pos($pos)
@@ -123,11 +126,11 @@ my class List does Positional { # declared in BOOTSTRAP
         # Get as many elements as we can.  If gimme stops before
         # reaching the end of the list, assume the list is infinite.
         my $n = self.gimme(*);
-        $!nextiter.defined ?? $Inf !! $n
+        $!nextiter.defined ?? Inf !! $n
     }
 
     method exists (\pos) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :exists adverb");
+        DEPRECATED("the :exists adverb with postcircumfix:<[ ]>");
         self.exists_pos(pos);
     }
     method exists_pos(\pos) {
@@ -140,7 +143,7 @@ my class List does Positional { # declared in BOOTSTRAP
         return unless self.DEFINITE;
         # loop through iterators until we have at least $n elements
         my int $count = nqp::elems(nqp::p6listitems(self));
-        my $eager = nqp::p6bool(nqp::istype($n, Whatever) || nqp::istype($n, Num) && $n == $Inf);
+        my $eager = nqp::p6bool(nqp::istype($n, Whatever) || nqp::istype($n, Num) && $n == Inf);
         while $!nextiter.defined && ($eager 
                                        ?? !$!nextiter.infinite 
                                        !! ($count < $n)) {
@@ -183,7 +186,7 @@ my class List does Positional { # declared in BOOTSTRAP
         ## of the list, resulting in an O(n) algorithm.
         my $elems = self.elems;
         return unless $elems;
-        $n = +$Inf if nqp::istype($n, Whatever);
+        $n = Inf if nqp::istype($n, Whatever);
         $n = $elems if $n > $elems;
         return self.at_pos($elems.rand.floor) if $n == 1;
         my Mu $rpa := nqp::clone($!items);
@@ -215,23 +218,43 @@ my class List does Positional { # declared in BOOTSTRAP
           !! fail 'Element shifted from empty list';
     }
 
+    multi method push(List:D: \value) {
+        if nqp::iscont(value) || !(nqp::istype(value, Iterable) || nqp::istype(value, Parcel)) {
+            $!nextiter.DEFINITE && self.gimme(*);
+            fail 'Cannot .push to an infinite list' if $!nextiter.defined;
+            nqp::p6listitems(self);
+            value.gimme(*) if nqp::istype(value, List); # fixes #121994
+            nqp::istype(value, self.of)
+                ?? nqp::push($!items, my $ = value)
+                !! X::TypeCheck.new(
+                      operation => '.push',
+                      expected  => self.of,
+                      got       => value,
+                    ).throw;
+            self
+        }
+        else {
+            callsame();
+        }
+    }
+    
     multi method push(List:D: *@values) {
         fail 'Cannot .push an infinite list' if @values.infinite;
-        self.gimme(*);
+        $!nextiter.DEFINITE && self.gimme(*);
         fail 'Cannot .push to an infinite list' if $!nextiter.defined;
         nqp::p6listitems(self);
 
         # don't bother with type checks
-        if ( self.of =:= Mu ) {
+        my $of := self.of;
+        if ( $of =:= Mu ) {
             nqp::push( $!items, @values.shift ) while @values.gimme(1);
         }
 
         # we must check types
         else {
-            my $of = self.of;
             while @values.gimme(1) {
                 my $value := @values.shift;
-                if $value ~~ $of {
+                if nqp::istype($value, $of) {
                     nqp::push( $!items, $value );
                 }
 
@@ -249,21 +272,39 @@ my class List does Positional { # declared in BOOTSTRAP
         self;
     }
 
+    multi method unshift(List:D: \value) {
+        if nqp::iscont(value) || !(nqp::istype(value, Iterable) || nqp::istype(value, Parcel)) {
+            nqp::p6listitems(self);
+            value.gimme(*) if nqp::istype(value, List); # fixes #121994
+            nqp::istype(value, self.of)
+                ?? nqp::unshift($!items, my $ = value)
+                !! X::TypeCheck.new(
+                      operation => '.push',
+                      expected  => self.of,
+                      got       => value,
+                    ).throw;
+            self
+        }
+        else {
+            callsame();
+        }
+    }
+    
     multi method unshift(List:D: *@values) {
         fail 'Cannot .unshift an infinite list' if @values.infinite;
         nqp::p6listitems(self);
 
         # don't bother with type checks
-        if ( self.of =:= Mu ) {
+        my $of := self.of;
+        if ( $of =:= Mu ) {
             nqp::unshift($!items, @values.pop) while @values;
         }
 
         # we must check types
         else {
-            my $of = self.of;
             while @values {
                 my $value := @values.pop;
-                if $value ~~ $of {
+                if nqp::istype($value, $of) {
                     nqp::unshift($!items, $value);
                 }
 
@@ -285,7 +326,7 @@ my class List does Positional { # declared in BOOTSTRAP
         my $elems = self.gimme(*);
         fail 'Cannot .roll from an infinite list' if $!nextiter.defined;
         return unless $elems;
-        $n = +$Inf if nqp::istype($n, Whatever);
+        $n = Inf if nqp::istype($n, Whatever);
         return self.at_pos($elems.rand.floor) if $n == 1;
 
         gather while $n > 0 {
@@ -489,6 +530,27 @@ my class List does Positional { # declared in BOOTSTRAP
         }, @.list;
     }
 
+    proto method rotor(|) {*}
+    multi method rotor(1, 0) { self }
+    multi method rotor($elems = 2, $overlap = 1) {
+        X::OutOfRange.new(
+            what => 'Overlap argument to List.rotor',
+            got  => $overlap,
+            range => (0 .. $elems - 1),
+        ).fail unless 0 <= $overlap < $elems;
+        X::OutOfRange.new(
+            what => 'Elements argument to List.rotor',
+            got  => $elems,
+            range => (0 .. *),
+        ).fail unless 0 <= $elems;
+
+        my $finished = 0;
+        gather while $finished + $overlap < self.gimme($finished + $elems) {
+            take item self[$finished ..^ $finished + $elems];
+            $finished += $elems - $overlap
+        }
+    }
+
     multi method gist(List:D:) { join ' ', map { $_.gist }, @(self) }
     multi method perl(List:D \SELF:) {
         self.gimme(*);
@@ -540,9 +602,9 @@ my class List does Positional { # declared in BOOTSTRAP
         fail('can only reduce with arity 2')
             unless &with.arity <= 2 <= &with.count;
         return unless self.DEFINITE;
-	my \vals = self.values;
+        my \vals = self.values;
         my Mu $val = vals.shift;
-	$val = with($val, $_) for vals;
+        $val = with($val, $_) for vals;
         $val;
     }
 
@@ -569,7 +631,8 @@ my class List does Positional { # declared in BOOTSTRAP
     }
 
     method permutations() {
-        gather take [self[@$_]] for permutations self.elems;
+        # need block on Moar because of RT#121830
+        gather { take [self[@$_]] for permutations self.elems }
     }
 }
 
@@ -589,8 +652,8 @@ proto infix:<xx>(|)       { * }
 multi infix:<xx>()        { fail "No zero-arg meaning for infix:<xx>" }
 multi infix:<xx>(Mu \x)   {x }
 multi infix:<xx>(Mu \x, $n is copy, :$thunked) {
-    $n = nqp::p6bool(nqp::istype($n, Whatever)) ?? $Inf !! $n.Int;
-    GatherIter.new({ take ($thunked ?? x.() !! x) while $n-- > 0; }, :infinite($n == $Inf)).list
+    $n = nqp::p6bool(nqp::istype($n, Whatever)) ?? Inf !! $n.Int;
+    GatherIter.new({ take ($thunked ?? x.() !! x) while --$n >= 0; }, :infinite($n == Inf)).list
 }
 
 proto sub pop(@) {*}
@@ -600,9 +663,11 @@ proto sub shift(@) {*}
 multi sub shift(@a) { @a.shift }
 
 proto sub unshift(|) {*}
+multi sub unshift(\a, \elem) { a.unshift: elem }
 multi sub unshift(\a, *@elems) { a.unshift: @elems }
 
 proto sub push(|) {*}
+multi sub push(\a, \elem) { a.push: elem }
 multi sub push(\a, *@elems) { a.push: @elems }
 
 sub reverse(*@a)            { @a.reverse }
