@@ -1,9 +1,18 @@
 my class Mix does Mixy {
     has Real $!total;
+    has $!WHICH;
+    has @!pairs;
 
-    method total { 
-        $!total //= [+] %!elems.values.map( { .value } ); 
-    }   
+    submethod WHICH {
+        $!WHICH //= self.^name
+          ~ '|'
+          ~ %!elems.keys.sort.map( { $_ ~ '(' ~ %!elems{$_}.value ~ ')' } );
+    }
+    method pairs() {
+        @!pairs ||= %!elems.values.map: { Enum.new(:key(.key),:value(.value)) };
+    }
+
+    method total (--> Real) { $!total //= [+] self.values }   
     method at_key($k --> Real) {
         my $key := $k.WHICH;
         %!elems.exists_key($key)
@@ -12,7 +21,7 @@ my class Mix does Mixy {
     }
 
     method delete ($a --> Real) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :delete adverb");
+        DEPRECATED('the :delete adverb with postcircumfix:<{ }>');
         self.delete_key($a);
     }
     method delete_key($a --> Real) is hidden_from_backtrace {
