@@ -30,31 +30,31 @@ my role Mixy does Baggy  {
     }
 
     method grab ($count = 1) {
-        fail ".grab is not supported on a {.self.^name}";
+        fail ".grab is not supported on a {self.^name}";
     }
 
     method pick ($count = 1) {
-        fail ".pick is not supported on a {.self.^name}";
+        fail ".pick is not supported on a {self.^name}";
     }
 
     method roll ($count = 1) {
-        my $total  = [+] self.values.grep: * > 0;
-        my $rolls  = $count ~~ Num ?? $total min $count !! $count;
+        my $total = [+] self.values.grep: * > 0;
+        my $rolls = $count ~~ Num
+          ?? $total min $count !! $count ~~ Whatever ?? Inf !! $count;
 #        my @pairs := %!elems.values;
         my @pairs := self.pairs;
 
-        map {
+        sub roll-one ($ignore?){
             my $rand = $total.rand;
             my $seen = 0;
             my $roll;
             for @pairs -> $pair {
-                next if ( $seen += $pair.value ) <= $rand;
-
-                $roll = $pair.key;
-                last;
+                return $pair.key if ( $seen += $pair.value ) > $rand;
             }
-            $roll;
-        }, 1 .. $rolls;
+        }
+        return roll-one if $rolls == 1;
+
+        map &roll-one, 1 .. $rolls;
     }
 }
 
