@@ -1,6 +1,7 @@
 my class X::Constructor::Positional { ... }
 my class X::Method::NotFound        { ... }
 my class X::Method::InvalidQualifier { ... }
+my class Obsolete { ... }
 
 my class Mu { # declared in BOOTSTRAP
     proto method ACCEPTS(|) { * }
@@ -47,8 +48,8 @@ my class Mu { # declared in BOOTSTRAP
     }
     
     proto method new(|) { * }
-    multi method new(*%attrinit) {
-        self.bless(|%attrinit);
+    multi method new(*%) {
+        nqp::invokewithcapture(nqp::findmethod(self, 'bless'), nqp::usecapture())
     }
     multi method new($, *@) {
         X::Constructor::Positional.new(:type( self )).throw();
@@ -59,7 +60,7 @@ my class Mu { # declared in BOOTSTRAP
     }
     
     method bless(*@autovivs, *%attrinit) {
-        if @autovivs && nqp::istype(@autovivs[0], Whatever) {
+        if @autovivs && nqp::istype(@autovivs.at_pos(0), Whatever) {
             DEPRECATED( "a call to bless without initial * parameter" );
             @autovivs.shift;
         }
@@ -119,8 +120,8 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 4) {
                 unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2)) {
-                    my $attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
-                    $attr = nqp::atpos($task, 3)(self, $attr);
+                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
+                    attr = nqp::atpos($task, 3)(self, attr);
                 }
             }
             elsif nqp::iseq_i($code, 5) {
@@ -194,8 +195,8 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 4) {
                 unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos($task, 2)) {
-                    my $attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
-                    $attr = nqp::atpos($task, 3)(self, $attr);
+                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
+                    attr = nqp::atpos($task, 3)(self, attr);
                 }
             }
             elsif nqp::iseq_i($code, 5) {
@@ -275,8 +276,8 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 4) {
                 unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2)) {
-                    my $attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
-                    $attr = nqp::atpos($task, 3)(self, $attr);
+                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos_s($task, 2));
+                    attr = nqp::atpos($task, 3)(self, attr);
                 }
             }
             elsif nqp::iseq_i($code, 5) || nqp::iseq_i($code, 6) || nqp::iseq_i($code, 7) {
@@ -354,8 +355,8 @@ my class Mu { # declared in BOOTSTRAP
             }
             elsif nqp::iseq_i($code, 4) {
                 unless nqp::attrinited(self, nqp::atpos($task, 1), nqp::atpos($task, 2)) {
-                    my $attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
-                    $attr = nqp::atpos($task, 3)(self, $attr);
+                    my \attr := nqp::getattr(self, nqp::atpos($task, 1), nqp::atpos($task, 2));
+                    attr = nqp::atpos($task, 3)(self, attr);
                 }
             }
             elsif nqp::iseq_i($code, 5) {
@@ -413,6 +414,7 @@ my class Mu { # declared in BOOTSTRAP
     method item(Mu \item:) is rw { item }
     
     proto method say(|) { * }
+    multi method say(Obsolete:D:) { say(self.gist) }
     multi method say() { say(self) }
     method print() { print(self) }
 
@@ -569,9 +571,10 @@ my class Mu { # declared in BOOTSTRAP
         $meth ??
             $meth(SELF, |c) !!
             X::Method::NotFound.new(
-                    method   => '!' ~ $name,
-                    typename => $type.HOW.name($type),
-                    :private,
+              invocant => SELF,
+              method   => '!' ~ $name,
+              typename => $type.HOW.name($type),
+              :private,
             ).throw;
     }
     
@@ -594,8 +597,9 @@ my class Mu { # declared in BOOTSTRAP
         my @result := SELF.dispatch:<.*>($name, |c);
         if @result.elems == 0 {
             X::Method::NotFound.new(
-                    method   => $name,
-                    typename => SELF.HOW.name(SELF),
+              invocant => SELF,
+              method   => $name,
+              typename => SELF.HOW.name(SELF),
             ).throw;
         }
         @result
@@ -621,7 +625,9 @@ my class Mu { # declared in BOOTSTRAP
     }
 
     method dispatch:<hyper>(Mu \SELF: $name, |c) {
-        hyper( -> \obj { obj."$name"(|c) }, SELF )
+        c
+            ?? hyper( -> \obj { obj."$name"(|c) }, SELF )
+            !! hyper( -> \obj { obj."$name"() }, SELF )
     }
     
     method WALK(:$name!, :$canonical, :$ascendant, :$descendant, :$preorder, :$breadth,
@@ -701,26 +707,26 @@ multi sub infix:<~~>(Mu \topic, Mu \matcher) {
     matcher.ACCEPTS(topic).Bool;
 }
 
-proto sub infix:<=:=>(Mu $a?, Mu $b?) { * }
-multi sub infix:<=:=>($a?)      { Bool::True }
+proto sub infix:<=:=>(Mu $?, Mu $?) { * }
+multi sub infix:<=:=>($?)      { Bool::True }
 multi sub infix:<=:=>(Mu \a, Mu \b) { 
     nqp::p6bool(nqp::eqaddr(a, b));
 }
 
 proto sub infix:<eqv>(Any $?, Any $?) { * }
-multi sub infix:<eqv>($a?)            { Bool::True }
+multi sub infix:<eqv>($?)            { Bool::True }
 multi sub infix:<eqv>(Any $a, Any $b) {
     $a.WHICH eq $b.WHICH
 }
 
 multi sub infix:<eqv>(@a, @b) {
-    unless @a.WHAT === @b.WHAT && @a.elems == @b.elems {
+    unless @a.WHAT === @b.WHAT && (my int $n = @a.elems) == @b.elems {
         return Bool::False
     }
-    for ^@a -> $i {
-        unless @a[$i] eqv @b[$i] {
-            return Bool::False;
-        }
+    my int $i = 0;
+    while $i < $n {
+        return Bool::False unless @a.at_pos($i) eqv @b.at_pos($i);
+        $i = $i + 1;
     }
     Bool::True
 }
