@@ -43,7 +43,7 @@ my class Num does Real { # declared in BOOTSTRAP
 
         my sub modf($num) { my $q = $num.Int; $num - $q, $q; }
 
-        (self == $Inf || self == -$Inf) && fail("Cannot coerce Inf to a Rat");
+        (self == Inf || self == -Inf) && fail("Cannot coerce Inf to a Rat");
         
         my Num $num = self;
         my Int $signum = $num < 0 ?? -1 !! 1;
@@ -322,17 +322,17 @@ multi infix:<**>(num $a, num $b) {
 
 
 multi infix:<cmp>(Num:D \a, Num:D \b) {
-    Order.(nqp::p6box_i(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b))))
+    ORDER(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
 multi infix:<cmp>(num $a, num $b) {
-    Order.(nqp::p6box_i(nqp::cmp_n($a, $b)))
+    ORDER(nqp::cmp_n($a, $b))
 }
 
 multi infix:«<=>»(Num:D \a, Num:D \b) {
-    Order.(nqp::p6box_i(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b))))
+    ORDER(nqp::cmp_n(nqp::unbox_n(a), nqp::unbox_n(b)))
 }
 multi infix:«<=>»(num $a, num $b) {
-    Order.(nqp::p6box_i(nqp::cmp_n($a, $b)))
+    ORDER(nqp::cmp_n($a, $b))
 }
 
 multi infix:<===>(Num:D \a, Num:D \b) {
