@@ -1,5 +1,5 @@
 my class Pair is Enum {
-    method key() is rw { nqp::getattr(self, Enum, '$!key') }
+    method key() { nqp::getattr(self, Enum, '$!key') }
     method value() is rw { nqp::getattr(self, Enum, '$!value') }
 
     multi method ACCEPTS(Pair:D: %h) {
