@@ -32,7 +32,7 @@ my role Rational[::NuT, ::DeT] does Real {
     method nude() { $!numerator, $!denominator }
     method Num() {
         $!denominator == 0
-          ?? ($!numerator < 0 ?? -$Inf !! $Inf)
+          ?? ($!numerator < 0 ?? -Inf !! Inf)
           !! nqp::p6box_n(nqp::div_In(
                 nqp::decont($!numerator),
                 nqp::decont($!denominator)
@@ -126,5 +126,4 @@ my role Rational[::NuT, ::DeT] does Real {
     }
 }
 
-
 # vim: ft=perl6 expandtab sw=4
