@@ -6,14 +6,14 @@ my class Regex { # declared in BOOTSTRAP
 
     multi method ACCEPTS(Regex:D \SELF: Mu \topic) {
         my $dollar_slash := nqp::getlexrelcaller(
-            nqp::ctxcaller(nqp::ctxcaller(nqp::ctx())),
+            nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
         $dollar_slash = SELF.(Cursor."!cursor_init"(topic, :c(0))).MATCH_SAVE;
     }
 
     multi method ACCEPTS(Regex:D \SELF: @a) {
         my $dollar_slash := nqp::getlexrelcaller(
-            nqp::ctxcaller(nqp::ctxcaller(nqp::ctx())),
+            nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
         for @a {
             $dollar_slash = SELF.(Cursor.'!cursor_init'($_, :c(0))).MATCH_SAVE;
@@ -23,7 +23,7 @@ my class Regex { # declared in BOOTSTRAP
     }
     multi method ACCEPTS(Regex:D \SELF: %h) {
         my $dollar_slash := nqp::getlexrelcaller(
-            nqp::ctxcaller(nqp::ctxcaller(nqp::ctx())),
+            nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
         for %h.keys {
             $dollar_slash = SELF.(Cursor.'!cursor_init'($_, :c(0))).MATCH_SAVE;
@@ -34,10 +34,10 @@ my class Regex { # declared in BOOTSTRAP
 
     multi method Bool(Regex:D:) {
         my $dollar_slash := nqp::getlexrelcaller(
-            nqp::ctxcaller(nqp::ctxcaller(nqp::ctx())),
+            nqp::ctxcallerskipthunks(nqp::ctx()),
             '$/');
         my $dollar_underscore := nqp::getlexrelcaller(
-            nqp::ctxcaller(nqp::ctxcaller(nqp::ctx())),
+            nqp::ctxcallerskipthunks(nqp::ctx()),
             '$_');
         $dollar_slash = $dollar_underscore.match(self);
         $dollar_slash.Bool()
