@@ -1,8 +1,9 @@
 my class Set does Setty {
     has Int $!total;
     has $!WHICH;
+    has @!pairs;
 
-    method total { $!total //= %!elems.elems }
+    method total (--> Int) { $!total //= %!elems.elems }
     submethod WHICH {
         $!WHICH := self.^name ~ '|' ~ %!elems.keys.sort if !$!WHICH.defined;
         $!WHICH
@@ -16,7 +17,7 @@ my class Set does Setty {
     }
 
     method delete ($a --> Bool) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :delete adverb");
+        DEPRECATED('the :delete adverb with postcircumfix:<{ }>');
         self.delete_key($a);
     }
     method delete_key($k --> Bool) is hidden_from_backtrace {
@@ -29,6 +30,12 @@ my class Set does Setty {
         X::Immutable.new( method => 'grabpairs', typename => self.^name ).throw;
     }
 
+    method pairs() {
+        @!pairs ||= %!elems.values.map: { Enum.new(:key($_),:value(True)) };
+    }
+
     method Set { self }
     method SetHash { SetHash.new(self.keys) }
 }
+
+# vim: ft=perl6 expandtab sw=4
