@@ -17,7 +17,7 @@ my class SetHash does Setty {
     }
 
     method delete($k) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :delete adverb");
+        DEPRECATED('the :delete adverb with postcircumfix:<{ }>');
         self.delete_key($k);
     }
     method delete_key($k --> Bool) {
