@@ -8,8 +8,10 @@ my role Setty does QuantHash {
     method kv { %!elems.values X, True }
     method elems(--> Int) { %!elems.elems }
     method total(--> Int) { %!elems.elems }
+    method minpairs(--> List) { self.pairs }
+    method maxpairs(--> List) { self.pairs }
     method exists ($k --> Bool) {  # is DEPRECATED doesn't work in settings
-        DEPRECATED("the :exists adverb");
+        DEPRECATED('the :exists adverb with postcircumfix:<{ }>');
         self.exists_key($k);
     }
     method exists_key($k --> Bool) {
@@ -64,7 +66,7 @@ my role Setty does QuantHash {
     }
 
     method list() { %!elems.values }
-    method pairs() { %!elems.values.map({ $_ => True }) } 
+    method pairs() { %!elems.values.map: { $_ => True } } 
     method grab($count = 1) {
         (%!elems{ %!elems.keys.pick($count) }:delete).list;
     }
