@@ -1,5 +1,6 @@
 my class X::Eval::NoSuchLang { ... }
 my class PseudoStash { ... }
+my class Label { ... }
 
 my &THROW :=
     -> | {
@@ -49,25 +50,37 @@ my &take := -> | {
     $parcel
 };
 
-my &last := -> | { 
-    my $parcel := 
-        &RETURN-PARCEL(nqp::p6parcel(nqp::p6argvmarray(), Nil));
-    THROW(nqp::decont($parcel), 
-          nqp::const::CONTROL_LAST) 
+my &last := -> | {
+    my Mu $args := nqp::p6argvmarray();
+    if nqp::islist($args) && nqp::istype(nqp::atpos($args, 0), Label) {
+        nqp::atpos($args, 0).last()
+    }
+    else {
+        my $parcel := nqp::decont(&RETURN-PARCEL(nqp::p6parcel($args, Nil)));
+        THROW($parcel, nqp::const::CONTROL_LAST)
+    }
 };
 
 my &next := -> | { 
-    my $parcel := 
-        &RETURN-PARCEL(nqp::p6parcel(nqp::p6argvmarray(), Nil));
-    THROW(nqp::decont($parcel), 
-          nqp::const::CONTROL_NEXT) 
+    my Mu $args := nqp::p6argvmarray();
+    if nqp::islist($args) && nqp::istype(nqp::atpos($args, 0), Label) {
+        nqp::atpos($args, 0).next()
+    }
+    else {
+        my $parcel := nqp::decont(&RETURN-PARCEL(nqp::p6parcel($args, Nil)));
+        THROW($parcel, nqp::const::CONTROL_NEXT)
+    }
 };
 
 my &redo := -> | { 
-    my $parcel := 
-        &RETURN-PARCEL(nqp::p6parcel(nqp::p6argvmarray(), Nil));
-    THROW(nqp::decont($parcel), 
-          nqp::const::CONTROL_REDO) 
+    my Mu $args := nqp::p6argvmarray();
+    if nqp::islist($args) && nqp::istype(nqp::atpos($args, 0), Label) {
+        nqp::atpos($args, 0).redo()
+    }
+    else {
+        my $parcel := nqp::decont(&RETURN-PARCEL(nqp::p6parcel($args, Nil)));
+        THROW($parcel, nqp::const::CONTROL_REDO)
+    }
 };
 
 my &succeed := -> | { 
@@ -147,31 +160,18 @@ multi sub warn(*@msg) is hidden_from_backtrace {
     0;
 }
 
-proto sub eval($, *%) {*}  # is DEPRECATED doesn't work in settings
-multi sub eval(Str $code, :$lang = 'perl6', PseudoStash :$context) {
-    DEPRECATED("'EVAL'");
-    my $eval_ctx := nqp::getattr(nqp::decont($context // CALLER::CALLER::), PseudoStash, '$!ctx');
-    my $?FILES   := 'eval_' ~ (state $no)++;
-    my $compiler := nqp::getcomp($lang);
-    X::Eval::NoSuchLang.new(:$lang).throw
-        if nqp::isnull($compiler);
-    my $compiled := $compiler.compile($code, :outer_ctx($eval_ctx), :global(GLOBAL));
-    nqp::forceouterctx(nqp::getattr($compiled, ForeignCode, '$!do'), $eval_ctx);
-    $compiled();
-}
 proto sub EVAL($, *%) {*}
-multi sub EVAL(Str $code, :$lang = 'perl6', PseudoStash :$context) {
-    my $eval_ctx := nqp::getattr(nqp::decont($context // CALLER::CALLER::), PseudoStash, '$!ctx');
-    my $?FILES   := 'eval_' ~ (state $no)++;
+multi sub EVAL(Cool $code, :$lang = 'perl6', PseudoStash :$context) {
+    my $eval_ctx := nqp::getattr(nqp::decont($context // CALLER::), PseudoStash, '$!ctx');
+    my $?FILES   := 'EVAL_' ~ (state $no)++;
     my $compiler := nqp::getcomp($lang);
     X::Eval::NoSuchLang.new(:$lang).throw
         if nqp::isnull($compiler);
-    my $compiled := $compiler.compile($code, :outer_ctx($eval_ctx), :global(GLOBAL));
+    my $compiled := $compiler.compile($code.Stringy, :outer_ctx($eval_ctx), :global(GLOBAL));
     nqp::forceouterctx(nqp::getattr($compiled, ForeignCode, '$!do'), $eval_ctx);
     $compiled();
 }
 
-
 sub exit($status = 0) {
     $_() for nqp::hllize(nqp::getcurhllsym('@END_PHASERS'));
     nqp::exit(nqp::unbox_i($status.Int));
@@ -216,13 +216,8 @@ sub shell($cmd) {
     $status
 }
 
-# XXX: Temporary definition of $Inf and $NaN until we have constants ava
-# need to come pretty early, because we use it in lots of setting files
-# constant Inf = ...
-# constant NaN = ...
-my $Inf = nqp::p6box_n(nqp::inf());
-my $NaN = nqp::p6box_n(nqp::nan());
-# EM 20130627 attempt at using constants failed during optimizing phase
+constant Inf = nqp::p6box_n(nqp::inf());
+constant NaN = nqp::p6box_n(nqp::nan());
 
 sub QX($cmd) {
 #?if parrot    
