@@ -3,32 +3,35 @@
 proto sub postcircumfix:<{ }>(|) { * }
 
 # %h<key>
-multi sub postcircumfix:<{ }>( \SELF, $key ) is rw {
-    SELF.at_key($key);
+multi sub postcircumfix:<{ }>( \SELF, \key ) is rw {
+    SELF.at_key(key);
 }
-multi sub postcircumfix:<{ }>(\SELF, $key, Mu :$BIND! is parcel) is rw {
-    SELF.bind_key($key, $BIND);
+multi sub postcircumfix:<{ }>(\SELF, \key, Mu \ASSIGN) is rw {
+    SELF.assign_key(key, ASSIGN);
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$SINK!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$SINK, |%other );
+multi sub postcircumfix:<{ }>(\SELF, \key, Mu :$BIND! is parcel) is rw {
+    SELF.bind_key(key, $BIND);
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$delete!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$delete, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$SINK!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$SINK, |%other );
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$exists!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$exists, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$delete!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$delete, |%other );
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$kv!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$kv, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$exists!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$exists, |%other );
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$p!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$p, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$kv!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$kv, |%other );
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$k!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$k, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$p!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$p, |%other );
 }
-multi sub postcircumfix:<{ }>( \SELF, $key, :$v!, *%other ) is rw {
-    SLICE_ONE( SELF, $key, False, :$v, |%other );
+multi sub postcircumfix:<{ }>( \SELF, \key, :$k!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$k, |%other );
+}
+multi sub postcircumfix:<{ }>( \SELF, \key, :$v!, *%other ) is rw {
+    SLICE_ONE( SELF, key, False, :$v, |%other );
 }
 
 # %h<a b c>
@@ -37,16 +40,21 @@ multi sub postcircumfix:<{ }>( \SELF, Positional \key ) is rw {
       ?? SELF.at_key(key) 
       !! key.map({ SELF{$_} }).eager.Parcel;
 }
+multi sub postcircumfix:<{ }>(\SELF, Positional \key, Mu \ASSIGN) is rw {
+    (nqp::iscont(key)
+      ?? SELF.at_key(key) 
+      !! key.map({ SELF{$_} }).eager.Parcel) = ASSIGN
+}
 multi sub postcircumfix:<{ }>(\SELF, Positional \key, :$BIND!) is rw {
     X::Bind::Slice.new(type => SELF.WHAT).throw;
 }
-multi sub postcircumfix:<{ }>(\SELF,Positional \key,:$SINK!,*%other) is rw {
+multi sub postcircumfix:<{ }>(\SELF,Positional \key, :$SINK!,*%other) is rw {
     SLICE_MORE( SELF, \key, False, :$SINK, |%other );
 }
-multi sub postcircumfix:<{ }>(\SELF,Positional \key,:$delete!,*%other) is rw {
+multi sub postcircumfix:<{ }>(\SELF,Positional \key, :$delete!,*%other) is rw {
     SLICE_MORE( SELF, \key, False, :$delete, |%other );
 }
-multi sub postcircumfix:<{ }>(\SELF,Positional \key,:$exists!,*%other) is rw {
+multi sub postcircumfix:<{ }>(\SELF,Positional \key, :$exists!,*%other) is rw {
     SLICE_MORE( SELF, \key, False, :$exists, |%other );
 }
 multi sub postcircumfix:<{ }>(\SELF, Positional \key, :$kv!, *%other) is rw {
@@ -66,6 +74,9 @@ multi sub postcircumfix:<{ }>(\SELF, Positional \key, :$v!, *%other) is rw {
 multi sub postcircumfix:<{ }>( \SELF, Whatever ) is rw {
     SELF{SELF.keys};
 }
+multi sub postcircumfix:<{ }>(\SELF, Whatever, Mu \ASSIGN) is rw {
+    SELF{SELF.keys} = ASSIGN;
+}
 multi sub postcircumfix:<{ }>(\SELF, Whatever, :$BIND!) is rw {
     X::Bind::Slice.new(type => SELF.WHAT).throw;
 }
