@@ -2,8 +2,6 @@ module Test::Util;
 
 use Test;
 
-# Tests for this testing code may be in the pugs repo under t/03-test-util/
-
 proto sub is_run(|) is export { * }
 
 # No input, no test name
@@ -63,6 +61,17 @@ multi sub is_run( Str $code, Str $input, %expected, Str $name, *%o ) {
     return;
 }
 
+our sub run( Str $code, Str $input = '', *%o) {
+    my %got = get_out( $code, $input, |%o );
+    if %got<err>:exists && %got<err>.chars {
+        diag 'error: ' ~ %got<err>;
+    }
+    if %got<test_died>:exists && %got<err>.chars {
+        diag 'test died: ' ~ %got<test_died>;
+    }
+    return %got<out>;
+}
+
 sub get_out( Str $code, Str $input?, :@args, :@compiler-args) is export {
     my $fnbase = 'getout';
     $fnbase ~= '-' ~ $*PID if defined $*PID;
@@ -76,8 +85,7 @@ sub get_out( Str $code, Str $input?, :@args, :@compiler-args) is export {
     };
 
     my @actual_args;
-    my $sep = q['];
-    $sep = q["] if $*OS ~~ /:i win/;
+    my $sep = $*DISTRO.is-win ?? q["] !! q['];
     for @args {
         if /<['"]>/ {
             die "Command line arguments may not contain single or double quotes";
@@ -118,43 +126,6 @@ sub get_out( Str $code, Str $input?, :@args, :@compiler-args) is export {
     return %out;
 }
 
-
-sub throws_like($code, $ex_type, *%matcher) is export {
-    my $msg;
-    if $code ~~ Callable {
-        $msg = 'code dies';
-        $code()
-    } else {
-        $msg = "'$code' died";
-        EVAL $code;
-    }
-    ok 0, $msg;
-    skip 'Code did not die, can not check exception', 1 + %matcher.elems;
-    CATCH {
-        default {
-            ok 1, $msg;
-            my $type_ok = $_ ~~ $ex_type;
-            ok $type_ok , "right exception type ({$ex_type.^name})";
-            if $type_ok {
-                for %matcher.kv -> $k, $v {
-                    my $got = $_."$k"();
-                    my $ok = $got ~~ $v,;
-                    ok $ok, ".$k matches {$v.defined ?? $v !! $v.gist}";
-                    unless $ok {
-                        diag "Got:      $got\n"
-                            ~"Expected: $v";
-                    }
-                }
-            } else {
-                diag "Got:      {$_.WHAT.gist}\n"
-                    ~"Expected: {$ex_type.gist}";
-                diag "Exception message: $_.message()";
-                skip 'wrong exception type', %matcher.elems;
-            }
-        }
-    }
-}
-
 =begin pod
 
 =head1 NAME
@@ -178,20 +149,6 @@ across Perl 6 implementations.
 
 =head1 FUNCTIONS
 
-=head2 throws_like($code, Mu $expected_type, *%matchers)
-
-If C<$code> is C<Callable>, calls it, otherwise C<EVAL>s it,
-and expects it thrown an exception.
-
-If an exception is thrown, it is compared to C<$expected_type>.
-
-Then for each key in C<%matchers>, a method of that name is called
-on the resulting exception, and its return value smart-matched against
-the value.
-
-Each step is counted as a separate test; if one of the first two fails,
-the rest of the tests are skipped.
-
 =head2 is_run( Str $code, Str $input?, %wanted, Str $name? )
 
 It runs the code given, feeding it the input given, and collects results
