@@ -303,7 +303,7 @@ sub gen_nqp {
             $impls{$b}{bin} = $bin;
             my %c = read_config($bin);
             my $nqp_have = $c{'nqp::version'} || '';
-            $impls{$b}{config} = \%c;
+            $impls{$b}{config} = \%c if %c;
             my $nqp_ok   = $nqp_have && cmp_rev($nqp_have, $nqp_want) >= 0;
             if ($nqp_ok) {
                 $impls{$b}{ok} = 1;
